// UDP pinger v0.3.0
// Copyright (C) 2009-2010 Stefan Ott <ott@iam.unibe.ch>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <sys/utsname.h>
#include <net/if.h>

#define DEFAULT_HOST "ff02::1"
#define DEFAULT_IFACE "wlan1"
#define PORT "4379"

#define VERSION "0.3.0"

static int
open_socket(char *host, int ifn)
{
	struct addrinfo hints, *ai;
	int gai;

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_INET6;
	hints.ai_socktype = SOCK_DGRAM;

	gai = getaddrinfo(host, PORT, &hints, &ai);

	if (gai)
	{
		perror("getaddrinfo");
		return -1;
	}

	struct addrinfo *cur = ai;
	int sockfd;


	while (cur) {
		sockfd = socket(cur->ai_family,
				cur->ai_socktype,
				cur->ai_protocol);

		if (setsockopt(sockfd, IPPROTO_IPV6, IPV6_MULTICAST_IF, &ifn,
			sizeof(ifn)) < 0)
		{
			perror("setsockopt");
			return EXIT_FAILURE;
		}


		if (!(sockfd < 0))
		{
			if (!connect(sockfd, cur->ai_addr, cur->ai_addrlen))
				break;

			close(sockfd);
			sockfd = -1;
		}
		cur = cur->ai_next;
	}
	freeaddrinfo(ai);

	return sockfd;
}

static int
is_gateway()
{
	FILE *file = fopen("/tmp/is_gateway", "r");

	if (file)
	{
		fclose(file);
		return 1;
	}
	return 0;
}

static int
ping(char *host, char *iface, int debug)
{
	int sockfd, gw, ifn = 0;
	struct utsname ugnm;

	syslog(LOG_NOTICE, "pinging host %s", host);
	if (debug > 0)
		printf("pinging host %s\n", host);

	if (uname(&ugnm) < 0)
	{
		perror("uname");
		return EXIT_FAILURE;
	}

	while (ifn == 0)
	{
		ifn = if_nametoindex(iface);

		if (ifn == 0)
		{
			if (debug)
				perror("if_nametoindex");
			printf("Interface %s not available (yet) - sleeping\n",
				iface);
			sleep(1);
		}
	}

	if (debug)
		printf("Using interface %s (%d)\n", iface, ifn);

	sockfd = open_socket(host, ifn);

	if (sockfd < 0)
	{
		perror("open_socket");
		return EXIT_FAILURE;
	}

	char payload[1024];
	while (1)
	{
		gw = is_gateway();
		snprintf(payload, 1023, "%s:%d", ugnm.nodename, gw);
		if (debug > 0)
			printf("%s (%d)\n", payload, (int) strlen(payload));

		write(sockfd, payload, strlen(payload));
		sleep(1);
	}

	close(sockfd);

	return EXIT_SUCCESS;
}

void
usage(char *self)
{
	printf("Usage: %s [-dh] [-t host]\n\n", self);
	printf("  -h           Show this help\n");
	printf("  -d           Debug\n");
	printf("  -v           Show version information\n");
	printf("  -t host      Ping host instead of the built-in default\n");
	printf("  -i iface     Use iface instead of the build-in default\n");
}

int
main(int argc, char **argv)
{
	int c;
	int debug = 0;
	char *host = DEFAULT_HOST;
	char *iface = DEFAULT_IFACE;

	while ((c = getopt(argc, argv, "dht:i:v")) != -1)
	{
		switch (c)
		{
			case 'd':
				debug = 1;
				break;
			case 'h':
				usage(argv[0]);
				return EXIT_SUCCESS;
			case 't':
				host = optarg;
				break;
			case 'i':
				iface = optarg;
				break;
			case 'v':
				printf("Pinger v%s\n", VERSION);
				return EXIT_SUCCESS;
				break;
			default:
				usage(argv[0]);
				return EXIT_FAILURE;
		}

	}

	if (debug == 0)
	{
		int i = fork();
		if (i < 0) {
			perror("fork");
			return EXIT_FAILURE;
		} else if (i > 0)
			return EXIT_SUCCESS;
	}
	return ping(host, iface, debug);
}
